
/*
	Deskbar button for SoundPlay.
	Copyright 1999 Marco Nelissen (marcone@xs4all.nl)

	This is using the preliminary/experimental user interface
	plugin API. Feedback on this API is greatly appreciated.

	Permission is hereby granted to use this code for creating other
	SoundPlay-plugins.
*/

#include <Application.h>
#include <Roster.h>
#include <Deskbar.h>
#include <AppFileInfo.h>
#include <PopUpMenu.h>
#include <MenuItem.h>
#include <stdio.h>
#include <string>

#include "SoundPlayDeskbar.h"
#include "BubbleHelper.h"

static const char *PLUGINID="sp-deskbar";
static const char *ADDON_SIGNATURE="application/x-vnd.marcone-soundplaydeskbar";
static const char *SOUNDPLAY_SIGNATURE="application/x-vnd.marcone-soundplay";

static void *getplugin(void **data,const char *name, const char *, uint32, plugin_info *, SoundPlayBufferedReader *reader);
static void destroyplugin(void*,void *_loader);
static status_t show(void*);
static void hide(void*);
static void setconfig(void*,BMessage *config);
static void getconfig(void*,BMessage *config);


static BubbleHelper *helper;

// =======================================================================
// standard plugin stuff
plugin_descriptor plugin={
	PLUGIN_INTERFACE_MAGIC,PLUGIN_INTERFACE_VERSION,
	PLUGINID,1, PLUGIN_IS_INTERFACE,

	"Deskbar button",
	"By Marco Nelissen.\n\n",
	NULL,
	NULL,
	&getplugin,
	&destroyplugin
};

static interface_plugin_ops plugin_ops={
	PLUGIN_FILTER_MAGIC, PLUGIN_FILTER_VERSION,
	show,
	hide,
	setconfig,
	getconfig
};

static plugin_descriptor *plugs[]={
	&plugin,
	0
};

plugin_descriptor **get_plugin_list(void) // this function gets exported
{
	return plugs;
}


// =======================================================================
// actual implementation

static void *getplugin(void **data,const char *name, const char *, uint32, plugin_info *info, SoundPlayBufferedReader *reader)
{
	*data=NULL;
	return &plugin_ops;
}

static void destroyplugin(void *,void *data)
{
	BDeskbar().RemoveItem("soundplay deskbar replicant");
}

static status_t show(void *data)
{
	DeskbarView *view=new DeskbarView();
	
	// in case something went wrong last time, remove
	// any previous soundplay deskbar replicants
	BDeskbar().RemoveItem("soundplay deskbar replicant");
	
	if(B_OK==BDeskbar().AddItem(view))
	{
		delete view;
		return B_OK;
	}

//	printf("couldn't add item to deskbar\n");
	return B_ERROR;
}

static void hide(void *data)
{
}

static void setconfig(void *data,BMessage *config)
{
}
static void getconfig(void *data,BMessage *config)
{
}


DeskbarView::DeskbarView()
	: BView(BRect(0,0,15,15),"soundplay deskbar replicant",B_WILL_DRAW,0)
{
	icon=NULL;
	pop=NULL;
	helper=new BubbleHelper();
}

DeskbarView::~DeskbarView()
{
	// I know I don't need to check for NULL, but I think it's clearer this way
	if(icon)
		delete icon;
	if(pop)
		delete pop;
	delete helper;
}

void DeskbarView::AttachedToWindow()
{
	SetHighColor(Parent()->ViewColor());
	SetDrawingMode(B_OP_OVER);
}

BMessenger DeskbarView::RemoteTrack(int32 index)
{
	BMessage question(B_GET_PROPERTY);
	question.AddSpecifier("Messenger");  // a track is a BHandler, and therefore supports the handler scripting suite
	question.AddSpecifier("track",index);
	BMessage answer;
	BMessenger remotetrack;

	if(B_OK == remoteapp.SendMessage(&question, &answer,2E6,2E6))
	{
		int32 error;
		if(B_OK == answer.FindInt32("error",&error) && error == B_OK && answer.HasMessenger("result"))
			answer.FindMessenger("result",&remotetrack);
	}
	return remotetrack;
}

BArchivable *DeskbarView::Instantiate(BMessage *archive)
{
	DeskbarView *view=new DeskbarView();
	view->remoteapp = BMessenger("application/x-vnd.marcone-soundplay");

	BMessage msg;
	if(B_OK==archive->FindMessage("bitmap",&msg))
		view->icon=new BBitmap(&msg);

	return view;
}

status_t DeskbarView::Archive(BMessage *message, bool deep) const
{
	BView::Archive(message,deep);
	message->AddString("add_on", ADDON_SIGNATURE);
	message->AddString("class","DeskbarView");
	message->AddMessenger("soundplay",this);

	// add the application's bitmap
	entry_ref ref;
	be_roster->FindApp(SOUNDPLAY_SIGNATURE,&ref);
	BFile file(&ref,B_READ_ONLY);
	BAppFileInfo ai(&file);
	BBitmap bitmap(BRect(0,0,15,15),B_COLOR_8_BIT);
	ai.GetIcon(&bitmap,B_MINI_ICON);
	BMessage msg(B_ARCHIVED_OBJECT);
	bitmap.Archive(&msg,true);
	message->AddMessage("bitmap",&msg);
//	printf("archiving\n");
	return B_OK;
}

void DeskbarView::Draw(BRect rect)
{
	if(icon)
	{
		FillRect(rect);
		DrawBitmap(icon,BPoint(0,0));
	}
}

void DeskbarView::MouseDown(BPoint point)
{
	if(!pop)
	{
		pop=new BPopUpMenu("deskbarmenu",false,false);
		pop->AddItem(new BMenuItem("Stop",new BMessage('stop')));
		pop->AddItem(new BMenuItem("Play",new BMessage('play')));
		pop->AddItem(new BMenuItem("Next",new BMessage('next')));
		pop->AddItem(new BMenuItem("Previous",new BMessage('prev')));
		pop->SetTargetForItems(this);
	}
	if(!pop->Window())
		pop->Go(ConvertToScreen(point),true,false,true);
}

void DeskbarView::MouseMoved(BPoint point, uint32 transit, const BMessage *message)
{
	if(transit == B_ENTERED_VIEW)
	{
		helper->SetHelp(this,NULL);
		BMessage question(B_GET_PROPERTY);
		question.AddSpecifier("info");
		BMessage answer;
		BMessenger track = RemoteTrack(0);
		if(track.IsValid())
		{
			if(B_OK==track.SendMessage(&question, &answer,2E6,2E6))
			{
				double time;
				double totaltime;
				const char *name;
				if(	B_OK==answer.FindDouble("abspos",&time) &&
					B_OK==answer.FindDouble("length",&totaltime) &&
					B_OK==answer.FindString("name",&name))
				{
					char buf[1024];
					buf[0] = '\0';
					if(answer.FindBool("unknownlength"))
					{
						sprintf(buf, "%s (%d:%02d/??:??",
							name, 
							int(time)/60,
							int(time)%60);
					}
					else
					{
						sprintf(buf, "%s (%d:%02d/%d:%02d",
							name, 
							int(time)/60,
							int(time)%60,
							int(totaltime)/60,
							int(totaltime)%60);
					}
					helper->SetHelp(this,buf);
				}
			}
		}
	}
}

void DeskbarView::MessageReceived(BMessage *mes)
{
	mes->PrintToStream();
	switch(mes->what)
	{
		case B_SIMPLE_DATA:
			{
				BMessenger track = RemoteTrack(0);
				if(!track.IsValid())
					track=remoteapp;

				if(mes->HasRef("refs"))
				{
					mes->what=B_REFS_RECEIVED;
					track.SendMessage(mes);
				}
			}
			break;

		case 'stop':
			{
				for(int i=0;;i++)
				{
					BMessenger track = RemoteTrack(i);
					if(!track.IsValid())
						break;
					BMessage command(B_SET_PROPERTY);
					command.AddFloat("data",0.0);
					command.AddSpecifier("pitch");
					track.SendMessage(&command, (BHandler*)NULL, 2E6);
				}
			}
			break;

		case 'play':
			{
				for(int i=0;;i++)
				{
					BMessenger track = RemoteTrack(i);
					if(!track.IsValid())
						break;
					BMessage command(B_SET_PROPERTY);
					command.AddFloat("data",1.0);
					command.AddSpecifier("pitch");
					track.SendMessage(&command, (BHandler*)NULL, 2E6);
				}
			}
			break;

		case 'prev':
			{
				for(int i=0;;i++)
				{
					BMessenger track = RemoteTrack(i);
					if(!track.IsValid())
						break;
					BMessage command(B_EXECUTE_PROPERTY);
					command.AddSpecifier("previous");
					track.SendMessage(&command, (BHandler*)NULL, 2E6);
				}
			}
			break;

		case 'next':
			{
				for(int i=0;;i++)
				{
					BMessenger track = RemoteTrack(i);
					if(!track.IsValid())
						break;
					BMessage command(B_EXECUTE_PROPERTY);
					command.AddSpecifier("next");
					track.SendMessage(&command, (BHandler*)NULL, 2E6);
				}
			}
			break;

	}
}


